using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;

namespace PlayWithYourFriends
{
    public partial class GameForm : Form
    {
        /// <summary>
        /// Device we are going to use to render our graphics
        /// </summary>
        private Device device;

        public GameForm()
        {
            InitializeComponent();
        }

        VertexBuffer vertBuffer;

        CustomVertex.PositionColored[] vertices;
        
        public void InitializeGraphics()
        {
            PresentParameters presentParams = new PresentParameters();
            presentParams.Windowed = true;
            presentParams.SwapEffect = SwapEffect.Discard;

            device = new Device(0, DeviceType.Default, this, CreateFlags.None, presentParams);

            // no need for lighting, we are setting the color of our triangle ourselves
            device.RenderState.Lighting = false;

            // Add the camera to our world and point it at the triangle
            device.Transform.View = Matrix.LookAtLH(
                new Vector3(0.0f, 0.0f, -5.0f), // camera position
                new Vector3(0.0f, 0.0f, 0.0f),  // camera target
                new Vector3(0.0f, 1.0f, 0.0f)   // camera up vector
                );

            // Set up the projection for our world
            device.Transform.Projection =
                Matrix.PerspectiveFovLH(
                (float)Math.PI / 4,     // field of view
                (float) this.Width / (float) this.Height,                   // aspect ratio
                1.0f,                   // near Z plane
                100.0f                  // far Z plane
                );

            // three vertices for a single triangle
            vertices = new CustomVertex.PositionColored[3];

            // Origin (0,0) is the middle of the screen 
            // The left hand edge is at X=-1, right hand edge at X=1
            // The top is y=1, the bottom is y=-1
            // This triangle will fill the screen

            vertices[0].X = 0.0f; 
            vertices[0].Y = 1.0f; 
            vertices[0].Z = 0.0f; 
            vertices[0].Color = Color.Red.ToArgb();

            vertices[1].X = 1.0f;
            vertices[1].Y = -1.0f; 
            vertices[1].Z = 0.0f; 
            vertices[1].Color = Color.Green.ToArgb();

            vertices[2].X = -1.0f; 
            vertices[2].Y = -1.0f; 
            vertices[2].Z = 0.0f; 
            vertices[2].Color = Color.Blue.ToArgb();

            // Create a vertex buffer to hold our triangle information
            vertBuffer = new VertexBuffer(
                typeof(CustomVertex.PositionColored),   // type of the buffer
                3,                                      // holding 3 vertices
                device,                                 // for our device
                Usage.WriteOnly,                        // never going to read it
                CustomVertex.PositionColored.Format,    // format required
                Pool.SystemMemory);                     // mobile 3D requires this

            // Set the data in the vertex buffer to our triangles
            vertBuffer.SetData(vertices, 0, LockFlags.None);

            timer1.Enabled = true;
        }

        // Do not want the system to paint the window background on refresh
        protected override void OnPaintBackground(PaintEventArgs e)
        {
        }

        // Called when the screen is redrawn
        protected override void OnPaint(PaintEventArgs e)
        {
            Render();
        }

        /// <summary>
        /// Our render method
        /// </summary>
        private void Render()
        {
            // Make the screen white to start with
            device.Clear(ClearFlags.Target, Color.White, 1.0f, 0);

            // Beginning of the draw process
            device.BeginScene();
            
            // Point the device at our vertex buffer

            device.SetStreamSource(0, vertBuffer, 0);

            // Ask the device to draw the contents of the buffer
            device.DrawPrimitives(PrimitiveType.TriangleList, 0, 1);

            // End of the draw process
            device.EndScene();

            // Now present the image on the screen
            device.Present();
        }

        private void doneMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void updateTriangle()
        {
            Text = "X:" + vertices[0].X + " Y:" + vertices[0].Y + " Z:" + vertices[0].Z;
            vertBuffer.SetData(vertices, 0, LockFlags.None);
            Invalidate();
        }


        private float step = 0.2f;

        private void moveLeft()
        {
            vertices[0].X -= step;
            vertices[1].X -= step;
            vertices[2].X -= step;
            updateTriangle();
        }

        private void moveRight()
        {
            vertices[0].X += step;
            vertices[1].X += step;
            vertices[2].X += step;
            updateTriangle();
        }

        private void moveUp()
        {
            vertices[0].Y += step;
            vertices[1].Y += step;
            vertices[2].Y += step;
            updateTriangle();
        }

        private void moveDown()
        {
            vertices[0].Y -= step;
            vertices[1].Y -= step;
            vertices[2].Y -= step;
            updateTriangle();
        }

        private void moveTowards()
        {
            vertices[0].Z -= step;
            vertices[1].Z -= step;
            vertices[2].Z -= step;
            updateTriangle();
        }

        private void moveAway()
        {
            vertices[0].Z += step;
            vertices[1].Z += step;
            vertices[2].Z += step;
            updateTriangle();
        }

        float rotateAngle = 0.0f;
        float rotateStep = 0.02f;

        private void redrawScene()
        {
            device.Transform.World = Matrix.RotationZ(rotateAngle);
            rotateAngle += rotateStep;
            Invalidate();
        }

        private void leftMenuItem_Click(object sender, EventArgs e)
        {
            moveLeft();
        }


        private void rightMenuItem_Click(object sender, EventArgs e)
        {
            moveRight();
        }


        private void upMenuItem_Click(object sender, EventArgs e)
        {
            moveUp();
        }

        private void downMenuItem_Click(object sender, EventArgs e)
        {
            moveDown();
        }

        private void towardsMenuItem_Click(object sender, EventArgs e)
        {
            moveTowards();
        }

        private void awayMenuItem_Click(object sender, EventArgs e)
        {
            moveAway();
        }

        private void GameForm_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == System.Windows.Forms.Keys.F1))
            {
                // Soft Key 1
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F2))
            {
                // Soft Key 2
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Up))
            {
                // Up
                moveUp();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Down))
            {
                // Down
                moveDown();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Left))
            {
                // Left
                moveLeft();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Right))
            {
                // Right
                moveRight();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
            {
                // Enter
                moveTowards();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D1))
            {
                // 1
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D2))
            {
                // 2
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D3))
            {
                // 3
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D4))
            {
                // 4
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D5))
            {
                // 5
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D6))
            {
                // 6
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D7))
            {
                // 7
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D8))
            {
                // 8
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D9))
            {
                // 9
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F8))
            {
                // *
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D0))
            {
                // 0
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F9))
            {
                // #
            }

        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            redrawScene();
        }
    }
}