using System;
using System.Drawing;
using System.Collections;

namespace PlayWithFriends
{
	/// <summary>
	/// Summary description for PlayField.
	/// </summary>
	public class PlayField
	{
		/// <summary>
		/// The rectangle to copy from the large bitmap.
		/// </summary>
		public Rectangle SourceRectangle ;

		/// <summary>
		/// The rectangle to copy into in the screen display bitmap.
		/// </summary>
		public Rectangle DestRectangle ;

		public int XOffset 
		{
			get 
			{
				return (int) xOffsetFloat;
			}
			set 
			{
				xOffsetFloat = (int) value;
			}
		}

		public int YOffset
		{
			get 
			{
				return (int) yOffsetFloat;
			}
			set 
			{
				yOffsetFloat = (int) value;
			}
		}

		private float xOffsetFloat;
		private float yOffsetFloat;

		public float Xspeed = 0 ;
		public float Yspeed = -1 ;

		/// <summary>
		///  A set of lists of sprites. Each of them is one part of the display
		///  Each is used for one layer of the screen.
		/// </summary>
		public ArrayList GameSpriteLists = new ArrayList() ;

		/// <summary>
		/// Clean copy of the background bitmap. 
		/// </summary>
		private Bitmap cleanBackgroundBitmap;

		/// <summary>
		/// Local copy of the background bitmap. 
		/// </summary>
		private Bitmap backgroundBitmap;

		/// <summary>
		/// Set property for the background bitmap. Updates the clear
		/// rectangle size to match the incoming image.
		/// </summary>
		public Bitmap BackgroundBitmap 
		{
			set 
			{
				backgroundBitmap = value;
			}
			get 
			{
				return backgroundBitmap;
			}
		}

		/// <summary>
		/// Clear the entire screen back to original.
		/// </summary>
		public void Clear()
		{
            using (Graphics g = Graphics.FromImage(backgroundBitmap))
            {
                g.DrawImage(cleanBackgroundBitmap, 0, 0);
            }
		}

		/// <summary>
		/// Updates the gameplay. Replaces the background behind each sprite, updates
		/// the sprite position and then asks the sprite to redraw itself.
		/// </summary>
		public void UpdateGame () 
		{
			using ( Graphics g = Graphics.FromImage ( backgroundBitmap ) )
			{
				// work through each arraylist updating the display

				// replace the background
				foreach ( ArrayList SpriteList in GameSpriteLists ) 
				{
					foreach ( BaseSprite sprite in SpriteList ) 
					{
						ReplaceBackground(g, sprite.SpriteRectangle);
					}
				}

				// update the sprites
				foreach ( ArrayList SpriteList in GameSpriteLists ) 
				{
					foreach ( BaseSprite sprite in SpriteList ) 
					{
						sprite.Update();
					}
				}

				xOffsetFloat += Xspeed;
				yOffsetFloat += Yspeed;

				if ( yOffsetFloat <= 0 ) 
				{
					YOffset = BackgroundBitmap.Height - DestRectangle.Height;
				}

				SourceRectangle.X = XOffset;
				SourceRectangle.Y = YOffset;

				// redraw the sprites
				foreach ( ArrayList SpriteList in GameSpriteLists ) 
				{
					foreach ( BaseSprite sprite in SpriteList ) 
					{
						sprite.SpriteRectangle.X = sprite.X + XOffset;
						sprite.SpriteRectangle.Y = sprite.Y + YOffset;
						sprite.Draw ( g );
					}
				}
			}
		}

		/// <summary>
		/// Replaces the background. Draws from the background bitmap.
		/// </summary>
		/// <param name="g">graphics for the draw destination</param>
		/// <param name="rect">destination rectangle</param>
		/// <summary>
		public void ReplaceBackground ( Graphics g, Rectangle rect ) 
		{
			g.DrawImage(cleanBackgroundBitmap, rect, rect, GraphicsUnit.Pixel);
		}

		/// <summary>
		/// Constructor. Stores the bitmap and builds the clear rectangle.
		/// </summary>
		/// <param name="inBackgroundBitmap">the bitmap to be used as the background</param>
		public PlayField( Bitmap inBackgroundBitmap, Rectangle inBoundingRectangle) 
		{
			BackgroundBitmap = new Bitmap ( inBackgroundBitmap );
			SourceRectangle = inBoundingRectangle;
			DestRectangle = inBoundingRectangle;
			cleanBackgroundBitmap = inBackgroundBitmap ;
			XOffset = 0 ;
			YOffset = BackgroundBitmap.Height - DestRectangle.Height;
		}

		public void Paint ( Graphics g ) 
		{
			g.DrawImage( backgroundBitmap, DestRectangle, SourceRectangle, GraphicsUnit.Pixel);
		}
	}
}
